@@ -198,13 +198,17 @@ enum
 #endif /* S_IXUSR */
 
 #if !defined(MAP_ANON) && defined(MAP_ANONYMOUS)
-#define MAP_ANON MAP_ANONYMOUS
+# define MAP_ANON MAP_ANONYMOUS
 #endif
 
 #ifndef MAP_FAILED
 # define MAP_FAILED ((void *)-1)
 #endif
 
+#if defined(HAVE_MMAP) && defined(MAP_ANON) && defined(MAP_PRIVATE)
+# define ENABLE_MMAP
+#endif
+
 /*
 SCO Open Server 3 has INADDR_LOOPBACK defined in rpc/rpc.h but
 including rpc/rpc.h breaks Solaris 6
@@ -1055,6 +1059,11 @@ struct winsize {
 #endif
 
 
+#ifdef HAVE_EVP_KEYMGMT_GET0_PROVIDER	/* OpenSSL >= 3.0.0 */
+# define ENABLE_KEY_PROVIDERS
+#endif
+
+
 #ifdef HAVE_EVP_KEM_FETCH		/* OpenSSL >= 3.0.0 */
 # define ENABLE_KEM_PROVIDERS
 #endif
