@@ -7,7 +7,7 @@
  * function compiled without _FORTIFY_SOURCE.
  */
 
-#include "config.h"
+#include "includes.h"
 
 #if defined(HAVE_FEATURES_H) && defined(_FORTIFY_SOURCE)
 # include <features.h>
@@ -15,7 +15,6 @@
 #  if __GLIBC_PREREQ(2, 15) && (_FORTIFY_SOURCE > 0)
 #   undef _FORTIFY_SOURCE
 #   undef __USE_FORTIFY_LEVEL
-#   include <sys/socket.h>
 void kludge_FD_SET(int, fd_set *);
 int kludge_FD_ISSET(int, fd_set *);
 
