@@ -28,9 +28,7 @@
 #include "includes.h"
 
 #include <sys/types.h>
-#include <sys/socket.h>
 #include <sys/wait.h>
-#include <sys/stat.h>
 
 #include <errno.h>
 #include <fcntl.h>
