@@ -29,7 +29,6 @@
 #include "includes.h"
 
 #include <sys/types.h>
-#include <sys/mman.h>
 #include <netinet/in.h>
 
 #include <errno.h>
@@ -130,7 +129,7 @@ sshkey_impl_from_type(int type)
 
 	for (p = keyimpls; *p != NULL; p++) {
 		if (!(*p)->enabled()) continue;
-		if ((*p)->type == type)
+		if ((*p)->id.type == type)
 			return *p;
 	}
 	return NULL;
@@ -143,8 +142,8 @@ sshkey_impl_from_type_nid(int type, int nid)
 
 	for (p = keyimpls; *p != NULL; p++) {
 		if (!(*p)->enabled()) continue;
-		if ((*p)->type == type &&
-		    ((*p)->nid == 0 || (*p)->nid == nid))
+		if ((*p)->id.type == type &&
+		    ((*p)->id.subtype == 0 || (*p)->id.subtype == nid))
 			return *p;
 	}
 	return NULL;
@@ -234,7 +233,7 @@ type_from_name(const char *name, int allow_short)
 		if ((strcmp((*p)->name, name) == 0) ||
 		    (allow_short && !(*p)->cert &&
 		    strcasecmp((*p)->shortname, name) == 0))
-			return (*p)->type;
+			return (*p)->id.type;
 	}
 }
 	return KEY_UNSPEC;
@@ -291,10 +290,10 @@ sshkey_ecdsa_nid_from_name(const char *name)
 
 	for (p = keyimpls; *p != NULL; p++) {
 		if (!(*p)->enabled()) continue;
-		if (!key_type_is_ecdsa_variant((*p)->type))
+		if (!key_type_is_ecdsa_variant((*p)->id.type))
 			continue;
 		if (strcmp((*p)->name, name) == 0)
-			return (*p)->nid;
+			return (*p)->id.subtype;
 	}
 }
 	return -1;
@@ -682,6 +681,7 @@ sshkey_new(int type)
 static int
 sshkey_prekey_alloc(u_char **prekeyp, size_t len)
 {
+#ifdef ENABLE_MMAP
 	u_char *prekey;
 
 	*prekeyp = NULL;
@@ -694,15 +694,23 @@ sshkey_prekey_alloc(u_char **prekeyp, size_t len)
 #  endif
 #endif
 	*prekeyp = prekey;
+#else
+	*prekeyp = calloc(1, len);
+#endif /* HAVE_MMAP et al */
 	return 0;
 }
 
 static void
 sshkey_prekey_free(void *prekey, size_t len)
 {
+#if defined(HAVE_MMAP) && defined(MAP_ANON) && defined(MAP_PRIVATE)
 	if (prekey == NULL)
 		return;
 	munmap(prekey, len);
+#else
+	UNUSED(len);
+	free(prekey);
+#endif /* HAVE_MMAP et al */
 }
 #endif /*def USE_SSHKEY_SHIELDING*/
 
@@ -2108,7 +2116,7 @@ sshkey_sigalg_by_name(const char *name)
 		if (!(*p)->cert)
 			return (*p)->name;
 		return sshkey_ssh_name_from_type_nid(
-		    sshkey_type_plain((*p)->type), (*p)->nid);
+		    sshkey_type_plain((*p)->id.type), (*p)->id.subtype);
 	}
 	return NULL;
 }
